package net.wanji.web.service.impl;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.gts.GeometryUtil;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.databus.dao.mapper.BaseAreaInfoMapper;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.po.BaseAreaInfoPO;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.web.bo.AddOrUpdateAreaBO;
import net.wanji.web.bo.AddOrUpdateJurisdictionBO;
import net.wanji.web.bo.PolygonBO;
import net.wanji.web.service.CrossManageService;
import net.wanji.web.vo.CrossIdAndNameVO;
import org.locationtech.jts.geom.Geometry;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/6/25 8:56
 */
@Service
@Slf4j
public class CrossManageServiceImpl implements CrossManageService {
    private final BaseCrossInfoMapper crossInfoMapper;
    private final BaseAreaInfoMapper baseAreaInfoMapper;

    public CrossManageServiceImpl(BaseCrossInfoMapper crossInfoMapper, BaseAreaInfoMapper baseAreaInfoMapper) {
        this.crossInfoMapper = crossInfoMapper;
        this.baseAreaInfoMapper = baseAreaInfoMapper;
    }

    @Override
    public List<CrossIdAndNameVO> selectCrossesByPolygon(PolygonBO polygonBO) {
        List<CrossIdAndNameVO> res = new ArrayList<>();
        String wkt = polygonBO.getWkt();
        List<BaseCrossInfoPO> crossList = crossInfoMapper.selectAll();

        for (BaseCrossInfoPO cross : crossList) {
            String location = cross.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            String crossLonLat = lonLat[0] + "," + lonLat[1];
            Geometry wktGeo = GeometryUtil.str2Geometry(wkt);
            Geometry crossGeo = GeometryUtil.str2Geometry(crossLonLat);
            // 判断点和多边形是否相交
            boolean isIntersect = wktGeo.intersects(crossGeo);
            if (isIntersect) {
                CrossIdAndNameVO crossIdAndNameVO = new CrossIdAndNameVO();
                crossIdAndNameVO.setCrossId(cross.getId());
                crossIdAndNameVO.setCrossName(cross.getName());
                res.add(crossIdAndNameVO);
            }
        }

        return res;
    }

    @Override
    @Transactional
    public void addOrUpdateArea(AddOrUpdateAreaBO addOrUpdateAreaBO) {
        Integer areaId = addOrUpdateAreaBO.getAreaId();
        String areaName = addOrUpdateAreaBO.getAreaName();
        String remark = addOrUpdateAreaBO.getRemark();
        List<String> crossIdList = addOrUpdateAreaBO.getCrossIdList();
        String wkt = addOrUpdateAreaBO.getWkt();

        if (ObjectUtil.isNotEmpty(areaId)) {
            // 修改
            // 修改子区名称和备注
            baseAreaInfoMapper.updateAreaNameAndRemark(areaName, remark, areaId);
            // 修改子区关联路口
            List<String> oldIds = crossInfoMapper.selectIdsByAreaId(areaId);
            crossInfoMapper.deleteAreaId(oldIds);
            crossInfoMapper.updateAreaId(crossIdList, areaId);
        } else {
            // 新增
            // 新增子区
            BaseAreaInfoPO baseAreaInfoPO = new BaseAreaInfoPO();
            baseAreaInfoPO.setCode(0);
            baseAreaInfoPO.setName(areaName);
            baseAreaInfoPO.setType(5);
            baseAreaInfoPO.setParentCode(0);
            baseAreaInfoPO.setLocation("");
            baseAreaInfoPO.setPolylines(wkt);
            baseAreaInfoPO.setRemark(remark);
            baseAreaInfoMapper.insertOne(baseAreaInfoPO);
            Integer newAreaId = baseAreaInfoPO.getId();
            // 关联子区路口
            crossInfoMapper.updateAreaId(crossIdList, newAreaId);
        }
    }

    @Override
    public void addOrUpdateJurisdiction(AddOrUpdateJurisdictionBO addOrUpdateJurisdictionBO) {
        Integer parentId = addOrUpdateJurisdictionBO.getParentId();
        Integer areaId = addOrUpdateJurisdictionBO.getAreaId();
        String areaName = addOrUpdateJurisdictionBO.getAreaName();
        String remark = addOrUpdateJurisdictionBO.getRemark();
        List<String> crossIdList = addOrUpdateJurisdictionBO.getCrossIdList();
        String wkt = addOrUpdateJurisdictionBO.getWkt();

        if (ObjectUtil.isNotEmpty(areaId)) {
            // 修改
            // 修改辖区名称和备注
            baseAreaInfoMapper.updateAreaNameAndRemark(areaName, remark, areaId);
            // 修改辖区关联路口
            List<String> oldIds = crossInfoMapper.selectIdsByAreaId(areaId);
            crossInfoMapper.deleteAreaId(oldIds);
            crossInfoMapper.updateAreaId(crossIdList, areaId);
        } else {
            // 新增
            // 新增子区
            BaseAreaInfoPO baseAreaInfoPO = new BaseAreaInfoPO();
            baseAreaInfoPO.setCode(0);
            baseAreaInfoPO.setName(areaName);
            baseAreaInfoPO.setType(5);
            baseAreaInfoPO.setParentCode(0);
            baseAreaInfoPO.setLocation("");
            baseAreaInfoPO.setPolylines(wkt);
            baseAreaInfoPO.setRemark(remark);
            baseAreaInfoMapper.insertOne(baseAreaInfoPO);
            Integer newAreaId = baseAreaInfoPO.getId();
            // 关联子区路口
            crossInfoMapper.updateAreaId(crossIdList, newAreaId);
        }
    }
}
